<?php

namespace Drupal\library_manager\Tests;

use Drupal\Core\Url;

/**
 * Tests the Library information interface.
 *
 * @group library_manager
 */
class LibraryInformationTest extends TestBase {

  /**
   * Tests collection page.
   */
  public function testCollectionPage() {
    $this->drupalGet('admin/structure/library');

    $this->assertPageTitle('Libraries');

    $this->assertByXpath('//input[@data-drupal-selector="library-filter" and @placeholder="Library name"]');

    $expected_header = [
      'Name',
      'Version',
      'License',
      'Definition',
      'Operations',
    ];
    $this->assertTableHeader('//main//table//th', $expected_header);

    // Check some arbitrary libraries in the list.
    $this->assertByXpath('//tr[td[1]/a[text() = "core/backbone"]][td[3][text() = "MIT"]]', 'Backbone library was found.');
    $this->assertByXpath('//tr[td[1]/a[text() = "system/drupal.system.modules"]][td[3][text() = "GNU-GPL-2.0-or-later"]]', 'Backbone library was found.');
    $this->assertByXpath('//tr[td[1]/a[text() = "library_manager/library_manager"]][td[3][text() = "GNU-GPL-2.0-or-later"]]', 'Backbone library was found.');

    // Check definition link.
    $this->clickLinkInRow('library_manager/alpha', 'alpha');
    $this->assertUrl('admin/structure/library/definition/alpha/edit');
    $this->assertPageTitle('Edit library definition');
  }

  /**
   * Tests details page.
   */
  public function testDetailsPage() {

    $this->drupalGet('admin/structure/library');
    $this->clickLinkInRow('library_manager_test/library_manager_test', 'library_manager_test/library_manager_test');
    $this->assertUrl('admin/structure/library/library/library_manager_test/library_manager_test');
    $this->assertPageTitle('library_manager_test');

    $items = $this->xpath('//div[contains(@class, "form-type-item")]');

    $data = [];
    foreach ($items as $item) {
      $data[(string) $item->label] = trim($item);
    }

    $expected_data = [
      'Name' => 'library_manager_test',
      'Extension' => 'library_manager_test',
      'Version' => '2.3.4',
      'License' => 'GNU-GPL-2.0-or-later',
    ];

    $this->assertEqual($data, $expected_data);

    $fieldsets = $this->xpath('//fieldset[contains(@class, "form-wrapper")]');

    // JS files.
    $this->assertEqual($fieldsets[0]->legend->span, 'JS', 'JS fieldset legend was found.');
    $js_items = $fieldsets[0]->div->ul->li;

    $file = 'modules/library_manager/tests/library_manager_test/js/example.js';
    $this->assertEqual($js_items[0]->a, $file, 'JS text was found.');
    $this->assertEqual($js_items[0]->a['href'], Url::fromUri('base://' . $file)->toString(), 'JS URL was found.');

    $file = 'libraries/example/global.js';
    $this->assertEqual($js_items[1]->a, $file, 'JS text was found.');
    $this->assertEqual($js_items[1]->a['href'], Url::fromUri('base://' . $file)->toString(), 'JS URL was found.');

    $file = 'http://example.com/external.js';
    $this->assertEqual($js_items[2]->a, $file, 'JS text was found.');
    $this->assertEqual($js_items[2]->a['href'], $file, 'JS URL was found.');

    // CSS files.
    $this->assertEqual($fieldsets[1]->legend->span, 'CSS', 'CSS fieldset legend was found.');
    $css_items = $fieldsets[1]->div->ul->li;

    $file = 'modules/library_manager/tests/library_manager_test/css/example.css';
    $this->assertEqual($css_items[0]->a, $file, 'CSS text was found.');
    $this->assertEqual($css_items[0]->a['href'], Url::fromUri('base://' . $file)->toString(), 'CSS URL was found.');

    $file = 'libraries/example/global.css';
    $this->assertEqual($css_items[1]->a, $file, 'CSS text was found.');
    $this->assertEqual($css_items[1]->a['href'], Url::fromUri('base://' . $file)->toString(), 'CSS URL was found.');

    $file = 'http://example.com/external.css';
    $this->assertEqual($css_items[2]->a, $file, 'CSS text was found.');
    $this->assertEqual($css_items[2]->a['href'], $file, 'CSS URL was found.');

    // Dependencies.
    $this->assertEqual($fieldsets[2]->legend->span, 'Dependencies', 'Dependencies legend was found.');
    $dependency_items = $fieldsets[2]->div->ul->li;
    $this->assertEqual((string) $dependency_items[0]->a, 'core/jquery');
    $this->assertEqual((string) $dependency_items[1]->a, 'core/drupal');
    $this->assertEqual((string) $dependency_items[2]->a, 'core/drupal.dialog.ajax');

    // Check if dependency link is valid.
    $this->clickLink('core/jquery');
    $this->assertPageTitle('jquery');

    // Check "Required by" section.
    $this->assertByXpath('//fieldset/legend/span[text() = "Required by"]');
    $details_link = $this->xpath('//fieldset/div/ul/li/a[text() = "library_manager_test/library_manager_test"]')[0];
    $this->drupalGet($this->getAbsoluteUrl($details_link['href']));
    $this->assertPageTitle('library_manager_test');
  }

  /**
   * Tests definition link.
   */
  public function testDefitionLink() {
    $this->drupalGet('admin/structure/library');
    $this->clickLinkInRow('alpha', 'library_manager/alpha');
    $this->assertPageTitle('alpha');
    $this->clickLink('alpha');
    $this->assertPageTitle('Edit library definition');
  }

  /**
   * Tests export page.
   */
  public function testExportPage() {
    $this->drupalGet('admin/structure/library');
    $this->clickLinkInRow('library_manager_test/library_manager_test', 'Export');
    $this->assertUrl('admin/structure/library/library/library_manager_test/library_manager_test/export');
    $this->assertPageTitle('library_manager_test');
    $this->assertByXpath('//textarea[@data-drupal-selector="library-export" and contains(text(), "version: 2.3.4")]');
  }

  /**
   * Tests autocomplete suggestions.
   */
  public function testAutocompletePage() {
    $this->drupalGet('admin/structure/library/autocomplete', ['query' => ['q' => 'backbone']]);
    $this->assertResponse(200);
    $this->assertHeader('Content-Type', 'application/json');
    $data = json_decode($this->getRawContent());
    $this->assertEqual($data[0]->label, 'core/backbone');
    $this->assertEqual($data[0]->value, 'core/backbone');
  }

}
